@@ -26,9 +26,7 @@ class H5(wnb.Base):
 
     def _get_group_zone_property_hash(self, group, zone_index):
 
-        properties = self._h5file[
-            "/" + group + "/Zone Properties/" + str(zone_index)
-        ]
+        properties = self._h5file["/" + group + "/Zone Properties/" + str(zone_index)]
 
         result = {}
 
@@ -115,8 +113,8 @@ class H5(wnb.Base):
             data["n"] = data["a"] - data["z"]
             data["source"] = nuclide_data[i][4].decode("ascii")
             data["state"] = nuclide_data[i][5].decode("ascii")
-            data["spin"] = nuclide_data[i][6]
-            data["mass excess"] = nuclide_data[i][7]
+            data["mass excess"] = nuclide_data[i][6]
+            data["spin"] = nuclide_data[i][7]
             result.append(data)
 
         return result
@@ -299,9 +297,7 @@ class H5(wnb.Base):
         for zone_labels in self.get_zone_labels_for_group(group):
             p = self._get_group_zone_property_hash(
                 group,
-                zone_labels_hash[
-                    (zone_labels[0], zone_labels[1], zone_labels[2])
-                ],
+                zone_labels_hash[(zone_labels[0], zone_labels[1], zone_labels[2])],
             )
             for property in properties:
                 result[property].append(p[property])
@@ -383,9 +379,7 @@ class H5(wnb.Base):
 
         self.set_plot_params(mpl, rcParams)
 
-        result = self.get_zone_properties_in_groups_as_floats(
-            zone, [prop1, prop2]
-        )
+        result = self.get_zone_properties_in_groups_as_floats(zone, [prop1, prop2])
 
         x = result[prop1] / xfactor
         y = result[prop2] / yfactor
@@ -441,10 +435,7 @@ class H5(wnb.Base):
 
         if plotParams:
             if len(plotParams) != len(species):
-                print(
-                    "Number of plotParam elements must equal"
-                    + " number of species."
-                )
+                print("Number of plotParam elements must equal" + " number of species.")
                 return
 
         plots = []
@@ -579,10 +570,7 @@ class H5(wnb.Base):
 
         if plotParams:
             if len(plotParams) != len(species):
-                print(
-                    "Number of plotParam elements must equal"
-                    + " number of species."
-                )
+                print("Number of plotParam elements must equal" + " number of species.")
                 return
 
         plots = []
@@ -685,9 +673,7 @@ class H5(wnb.Base):
 
         if yfactor:
             if len(yfactor) != len(species):
-                print(
-                    "yfactor length must be the same as the number of species."
-                )
+                print("yfactor length must be the same as the number of species.")
                 return
         else:
             yfactor = np.full(len(species), 1.0)
@@ -729,7 +715,7 @@ class H5(wnb.Base):
     def make_mass_fractions_movie(
         self,
         species,
-        movie_name,
+        movie_name="",
         property=None,
         fps=15,
         xfactor=1,
@@ -747,7 +733,7 @@ class H5(wnb.Base):
             in the movie.
 
             ``movie_name`` (:obj:`str`): A string giving the name of the
-            movie to be made.
+            resultin movie file.
 
             ``property`` (:obj:`str`, optional): A string giving property
             to be the x axis.  Defaults to zone index.
@@ -787,15 +773,12 @@ class H5(wnb.Base):
             Include directly, as a :obj:`dict`, or both.
 
         Returns:
-            A matplotlib plot.
+            The animation.
 
         """
         if plotParams:
             if len(plotParams) != len(species):
-                print(
-                    "Number of plotParam elements must equal"
-                    + " number of species."
-                )
+                print("Number of plotParam elements must equal" + " number of species.")
                 return
 
         fig = plt.figure()
@@ -827,9 +810,7 @@ class H5(wnb.Base):
                         groups[i], [property]
                     )
                     plt.plot(
-                        p_prop[property] / xfactor,
-                        x[:, nuclide_data[sp]["index"]],
-                        **p
+                        p_prop[property] / xfactor, x[:, nuclide_data[sp]["index"]], **p
                     )
                 else:
                     plt.plot(x[:, nuclide_data[sp]["index"]], **p)
@@ -837,9 +818,9 @@ class H5(wnb.Base):
             if title_func:
                 tf = title_func(i)
                 if tf:
-                    if type(tf) is tuple:
+                    if isinstance(tf, tuple):
                         plt.title(tf[0], **tf[1])
-                    elif type(tf) is str:
+                    elif isinstance(tf, str):
                         plt.title(tf)
                     else:
                         print("Invalid return from title function.")
@@ -857,4 +838,7 @@ class H5(wnb.Base):
             plt.draw()
 
         anim = animation.FuncAnimation(fig, updatefig, len(groups))
-        anim.save(movie_name, fps=fps)
+        if movie_name:
+            anim.save(movie_name, fps=fps)
+
+        return anim
