@@ -145,7 +145,8 @@ class Reaction(wb.Base):
             return self._get_user_rate_data(user_rate[0])
 
     def _set_data(self, reaction_node):
-        self.source = reaction_node.xpath("source")[0].text
+        if reaction_node.xpath("source"):
+            self.source = reaction_node.xpath("source")[0].text
 
         reactants = reaction_node.xpath("reactant")
         for reactant in reactants:
@@ -157,11 +158,16 @@ class Reaction(wb.Base):
 
         self.data = self._get_reaction_data(reaction_node)
 
-    def _compute_rate_table_rate(self, t9):
+    def _compute_rate_table_rate_interpolation(self, t9):
         t = self.data["t9"]
         lr = np.log10(self.data["rate"])
         sef = self.data["sef"]
 
+        if t9 < t[0]:
+            return np.power(10.0, lr[0]) * sef[0]
+        elif t9 > t[len(t) - 1]:
+            return np.power(10.0, lr[len(t) - 1]) * sef[len(t) - 1]
+
         if len(t) <= 2:
             f1 = interp1d(t, lr, kind="linear")
             f2 = interp1d(t, sef, kind="linear")
@@ -171,6 +177,14 @@ class Reaction(wb.Base):
             f2 = interp1d(t, sef, kind="cubic")
             return np.power(10.0, f1(t9)) * f2(t9)
 
+    def _compute_rate_table_rate(self, t9):
+        if isinstance(t9, float):
+            return self._compute_rate_table_rate_interpolation(t9)
+        else:
+            return np.array(
+                [self._compute_rate_table_rate_interpolation(x) for x in t9]
+            )
+
     def _compute_non_smoker_fit_rate_for_fit(self, fit, t9):
         def non_smoker_function(fit, t9):
             x = (
@@ -231,55 +245,20 @@ class Reaction(wb.Base):
             print("No such reaction type")
             return None
 
-    def get_reactants(self):
-        """Method to return the reactants for a reaction.
-
-        Returns:
-            :obj:`list`: A list of :obj:`str` giving the reactants.
-
-        """
-        return self.reactants
-
-    def get_products(self):
-        """Method to return the products for a reaction.
-
-        Returns:
-            :obj:`list`: A list of :obj:`str` giving the products.
-
-        """
-        return self.products
-
-    def get_source(self):
-        """Method to return the data source for a reaction.
-
-        Returns:
-            :obj:`str`: The source.
-
-        """
-        return self.source
-
-    def get_data(self):
-        """Method to return the data for a reaction.
-
-        Returns:
-            :obj:`dict`: A dictionary containing the rate data for the
-            reaction.
-
-        """
-        return self.data
-
-    def get_string(self):
-        """Method to return the string for a reaction.
-
-        Returns:
-            :obj:`str`: The reaction string.
-
-        """
-
-        s = " + ".join(self.reactants)
-        s += " -> "
-        s += " + ".join(self.products)
-        return s
+    def _get_reactant_and_product_xpath(self):
+        reactants = []
+        for r in self.reactants:
+            reactants.append(r)
+        products = []
+        for p in self.products:
+            products.append(p)
+        result = "[reactant = '" + reactants[0] + "'"
+        for i in range(1, len(reactants)):
+            result += " and reactant = '" + reactants[i] + "'"
+        for p in products:
+            result += " and product = '" + p + "'"
+        result += "]"
+        return result
 
     def get_latex_string(self):
         """Method to return the latex string for a reaction.
@@ -304,6 +283,29 @@ class Reaction(wb.Base):
         s += "$"
         return s
 
+    def get_data(self):
+        """Method to return the data for a reaction.
+
+        Returns:
+            :obj:`dict`: A dictionary containing the rate data for the
+            reaction.
+
+        """
+        return self.data
+
+    def get_string(self):
+        """Method to return the string for a reaction.
+
+        Returns:
+            :obj:`str`: The reaction string.
+
+        """
+
+        s = " + ".join(self.reactants)
+        s += " -> "
+        s += " + ".join(self.products)
+        return s
+
 
 class Xml(wb.Base):
     """A class for reading and plotting webnucleo xml files.
@@ -317,7 +319,8 @@ class Xml(wb.Base):
        """
 
     def __init__(self, file):
-        self._xml = etree.parse(file)
+        parser = etree.XMLParser(remove_blank_text=True)
+        self._xml = etree.parse(file, parser)
         self._root = self._xml.getroot()
 
     def _get_state_data(self, state_data, node):
@@ -368,19 +371,17 @@ class Xml(wb.Base):
 
         return result
 
-    def _get_max_nucleon_numbers(self):
-        nd = self._get_nuclide_data_array("")
+    def get_type(self):
+        """Method to retrieve the root type of the webnucleo XML.
 
-        z = np.zeros(len(nd), dtype=np.int_)
-        n = np.zeros(len(nd), dtype=np.int_)
-        a = np.zeros(len(nd), dtype=np.int_)
+        Returns:
+            :obj:`str`: One of `nuclear_data`, `reaction_data`,
+            `nuclear_network`, `zone_data`, `libnucnet_input` indicating
+            the root type of the XML.
 
-        for i in range(len(nd)):
-            z[i] = nd[i]["z"]
-            n[i] = nd[i]["n"]
-            a[i] = nd[i]["a"]
+        """
 
-        return {"z": np.amax(z), "n": np.amax(n), "a": np.amax(a)}
+        return self._root.tag
 
     def get_nuclide_data(self, nuc_xpath=" "):
         """Method to retrieve nuclear data from webnucleo XML.
@@ -458,12 +459,14 @@ class Xml(wb.Base):
         species = zone.xpath("mass_fractions/nuclide")
 
         for sp in species:
-            data = {}
-            data["z"] = int((sp.xpath("z"))[0].text)
-            data["a"] = int((sp.xpath("a"))[0].text)
-            data["n"] = data["a"] - data["z"]
-            data["x"] = float((sp.xpath("x"))[0].text)
-            result[sp.xpath("@name")[0]] = data
+            z = int((sp.xpath("z"))[0].text)
+            a = int((sp.xpath("a"))[0].text)
+            name_array = sp.xpath("@name")
+            if len(name_array) == 0:
+                name = self.create_nuclide_name(z, a, "")
+            else:
+                name = name_array[0]
+            result[(name, z, a)] = float((sp.xpath("x"))[0].text)
 
         return result
 
@@ -528,9 +531,7 @@ class Xml(wb.Base):
 
         for i in range(len(zones)):
             for sp in species:
-                data = zones[i].xpath(
-                    'mass_fractions/nuclide[@name="%s"]/x' % sp
-                )
+                data = zones[i].xpath('mass_fractions/nuclide[@name="%s"]/x' % sp)
                 if len(data) == 1:
                     result[sp][i] = float(data[0].text)
 
@@ -555,7 +556,7 @@ class Xml(wb.Base):
         properties_t = {}
 
         for property in properties:
-            if type(property) is str:
+            if isinstance(property, str):
                 properties_t[property] = (property,)
             else:
                 properties_t[property] = property
@@ -595,15 +596,33 @@ class Xml(wb.Base):
                 data = zone.xpath(path)
 
                 if len(data) == 0:
-                    print(
-                        "Property", self._get_property_name(tup), "not found."
-                    )
+                    print("Property", self._get_property_name(tup), "not found.")
                     return
 
                 dict[property].append(data[0].text)
 
         return dict
 
+    def _get_all_zone_properties(self, zone):
+        result = {}
+
+        props = zone.xpath("optional_properties/property")
+
+        for property in props:
+            p_name = ""
+            name = property.xpath("@name")
+            tag1 = property.xpath("@tag1")
+            tag2 = property.xpath("@tag2")
+            if tag1:
+                p_name = (name[0], tag1[0])
+                if tag2:
+                    p_name += (tag2[0],)
+            else:
+                p_name = name[0]
+            result[p_name] = property.text
+
+        return result
+
     def get_all_properties_for_zone(self, zone_xpath):
         """Method to retrieve all properties in a zone in an xml file
 
@@ -625,22 +644,7 @@ class Xml(wb.Base):
             print("Incorrect number of zones.")
             return
 
-        props = zones[0].xpath("optional_properties/property")
-
-        for property in props:
-            p_name = ""
-            name = property.xpath("@name")
-            tag1 = property.xpath("@tag1")
-            tag2 = property.xpath("@tag2")
-            if tag1:
-                p_name = (name[0], tag1[0])
-                if tag2:
-                    p_name += (tag2[0],)
-            else:
-                p_name = name[0]
-            result[p_name] = property.text
-
-        return result
+        return self._get_all_zone_properties(zones[0])
 
     def get_properties_as_floats(self, properties, zone_xpath=" "):
         """Method to retrieve properties in zones in an xml file as floats.
@@ -685,15 +689,16 @@ class Xml(wb.Base):
 
         zones = self._get_zones(zone_xpath)
 
-        m = self._get_max_nucleon_numbers()
+        lim = self.get_network_limits()
+        z_max = np.max(lim["z"])
+        n_max = np.max(lim["n_max"])
 
-        result = np.zeros((len(zones), m["z"] + 1, m["n"] + 1))
+        result = np.zeros((len(zones), z_max + 1, n_max + 1))
 
         for i in range(len(zones)):
             sp = self._get_nuclide_data_for_zone(zones[i])
-
             for s in sp:
-                result[i, sp[s]["z"], sp[s]["n"]] += sp[s]["x"] / sp[s]["a"]
+                result[i, s[1], s[2] - s[1]] += sp[s] / s[2]
 
         return result
 
@@ -718,17 +723,20 @@ class Xml(wb.Base):
             print("nucleon must be 'z', 'n', or 'a'.")
             return
 
-        zones = self._get_zones(zone_xpath)
+        y = self.get_all_abundances_in_zones(zone_xpath)
 
-        m = self._get_max_nucleon_numbers()
-
-        result = np.zeros((len(zones), m[nucleon] + 1))
-
-        for i in range(len(zones)):
-            sp = self._get_nuclide_data_for_zone(zones[i])
-
-            for s in sp:
-                result[i, sp[s][nucleon]] += sp[s]["x"] / sp[s]["a"]
+        if nucleon == "z":
+            result = np.sum(y, axis=2)
+        elif nucleon == "n":
+            result = np.sum(y, axis=1)
+        else:
+            yz = np.sum(y, axis=2)
+            yn = np.sum(y, axis=1)
+            result = np.zeros((y.shape[0], y.shape[1] + y.shape[2] + 2))
+            for i in range(y.shape[0]):
+                for i_z in range(y.shape[1]):
+                    for i_n in range(y.shape[2]):
+                        result[i, i_z + i_n] += y[i, i_z, i_n]
 
         return result
 
@@ -846,10 +854,7 @@ class Xml(wb.Base):
 
         if plotParams:
             if len(plotParams) != len(species):
-                print(
-                    "Number of plotParam elements must equal number"
-                    + " of species."
-                )
+                print("Number of plotParam elements must equal number" + " of species.")
                 return
 
         plots = []
@@ -936,9 +941,7 @@ class Xml(wb.Base):
 
         if plotParams:
             if y.shape[0] != len(plotParams):
-                print(
-                    "Number of plotParam elements must equal number of plots."
-                )
+                print("Number of plotParam elements must equal number of plots.")
                 return
 
         for i in range(y.shape[0]):
@@ -963,30 +966,77 @@ class Xml(wb.Base):
 
         self.show_or_close(plt, kwargs)
 
-    def make_abundances_vs_nucleon_number_movie(
+    def get_chain_abundances(self, nucleon, zone_xpath="", vs_A=False):
+        """Method to retrieve the abundances in a chain (fixed Z or N).
+
+        Args:
+
+            ``nucleon`` (:obj:`tuple`, optional): A tuple giving the nucleon.
+            The first entry must be the nucleon type (must be 'z' or 'n') while
+            the second entry must be the value.
+
+            ``zone_xpath`` (:obj:`str`, optional): A string giving the XPath
+            expression to select the zones. Defaults to all zones.
+
+            ``vs_A`` (:obj:`bool`, optional): A boolean to select whether
+            abscissa data should be mass number.
+
+        Returns:
+            A :obj:`tuple` containing an array of the nucleon values as the
+            first element and a two-d :obj:`numpy.array` as the second element.
+            The first index of the two-d array indicates the step and the
+            second the abundance of the species with the corresponding nucleon
+            number in the first element with the same index.
+
+        """
+        abunds = self.get_all_abundances_in_zones(zone_xpath=zone_xpath)
+        if nucleon[0] == "z":
+            x = range(abunds.shape[2])
+        elif nucleon[0] == "n":
+            x = range(abunds.shape[1])
+        else:
+            print("Invalid nucleon")
+            return None
+        if vs_A:
+            x = [xx + nucleon[1] for xx in x]
+        if nucleon[0] == "z":
+            y = abunds[:, nucleon[1], :]
+        elif nucleon[0] == "n":
+            y = abunds[:, :, nucleon[1]]
+
+        return (x, y)
+
+    def make_abundance_chain_movie(
         self,
-        movie_name,
-        nucleon="a",
-        zone_xpath=" ",
+        movie_name=None,
+        nucleon=("z", 26),
+        zone_xpath="",
+        plot_vs_A=False,
         fps=15,
         title_func=None,
         rcParams=None,
         plotParams=None,
+        extraFixedCurves=None,
+        extraCurves=None,
         **kwargs
     ):
-        """Method to make of movie of abundances summed by nucleon number.
+        """Method to make of movie of abundances in a chain (fixed Z or N).
 
         Args:
 
-            ``movie_name`` (:obj:`str`): A string giving the name of the
-            movie to be made.
+            ``movie_name`` (:obj:`str`, optional): A string giving the name of
+            resulting movie file.
 
-            ``nucleon`` (:obj:`str`, optional): A string giving the nucleon
-            (must be 'z', 'n', or 'a').  Defaults to 'a'.
+            ``nucleon`` (:obj:`tuple`, optional): A tuple giving the nucleon.
+            The first entry must be the nucleon type (must be 'z' or 'n') while
+            the second entry must be the value.
 
             ``zone_xpath`` (:obj:`str`, optional): A string giving the XPath
             expression to select the zones. Defaults to all zones.
 
+            ``plot_vs_A`` (:obj:`bool`, optional): A boolean to select whether
+            abscissa should be mass number.
+
             ``fps`` (:obj:`float`, optional): A float giving the frames
             per second in the resulting movie.
 
@@ -1013,11 +1063,184 @@ class Xml(wb.Base):
             dictionaries of valid :obj:`matplotlib.pyplot.plot` optional
             keyword arguments to be applied to the lines in the movie.
 
+            ``extraFixedCurves`` (:obj:`list`, optional): A list of
+            :obj:`tuple` objects giving fixed curves to appear on each
+            frame of the animation.  The first element of the tuple is a
+            :obj:`list` giving the abscissa values for the curve, the second
+            element is the ordinate values for the curve, and the third
+            element, if present, is a :obj:`dict` of
+            :obj:`matplotlib.pyplot.plot` optional keyword arguments to be
+            applied to the extra fixed curves in the movie.
+
+            ``extraCurves`` (:obj:`list`, optional): A list of
+            :obj:`tuple` objects giving curves to appear on each
+            frame of the animation.  The first element of the tuple is a
+            :obj:`list` giving the abscissa values for the curve, the second
+            element is a two-d :obj:`numpy.array` giving the ordinate values
+            for the curve corresponding to each timestep in the animation,
+            and the third element, if present, is a :obj:`dict` of
+            :obj:`matplotlib.pyplot.plot` optional keyword arguments to be
+            applied to the extra fixed curves in the movie.
+
+
             ``**kwargs``:  Acceptable :obj:`matplotlib.pyplot` functions.
             Include directly, as a :obj:`dict`, or both.
 
         Returns:
-            A matplotlib plot.
+            The animation.
+
+        """
+        fig = plt.figure()
+
+        self.set_plot_params(mpl, rcParams)
+
+        x, y = self.get_chain_abundances(nucleon, zone_xpath=zone_xpath, vs_A=plot_vs_A)
+        props = self.get_properties_as_floats(
+            ["time", "t9", "rho"], zone_xpath=zone_xpath
+        )
+
+        # Check the array length against the number of steps
+
+        if extraCurves:
+            for tup in extraCurves:
+                if tup[1].shape[0] != y.shape[0]:
+                    print("Extra curve does not have the right length.")
+                    return None
+
+        def updatefig(i):
+            fig.clear()
+
+            if plotParams:
+                plt.plot(x, y[i], **plotParams)
+            else:
+                plt.plot(x, y[i])
+
+            if extraFixedCurves:
+                for tup in extraFixedCurves:
+                    if len(tup) == 2:
+                        plt.plot(tup[0], tup[1])
+                    else:
+                        plt.plot(tup[0], tup[1], **tup[2])
+
+            if extraCurves:
+                for tup in extraCurves:
+                    if len(tup) == 2:
+                        plt.plot(tup[0], tup[1][i])
+                    else:
+                        plt.plot(tup[0], tup[1][i], **tup[2])
+
+            if title_func:
+                tf = title_func(i)
+                if tf:
+                    if isinstance(tf, tuple):
+                        plt.title(tf[0], **tf[1])
+                    elif isinstance(tf, str):
+                        plt.title(tf)
+                    else:
+                        print("Invalid return from title function.")
+                        return
+            else:
+                if nucleon[0] == "z":
+                    pre_str = "Z = {0:d}, ".format(nucleon[1])
+                else:
+                    pre_str = "N = {0:d}, ".format(nucleon[1])
+                title_str = pre_str + self.make_time_t9_rho_title_str(props, i)
+                plt.title(title_str)
+            if "xlabel" not in kwargs:
+                if not plot_vs_A:
+                    if nucleon[0] == "z":
+                        plt.xlabel("N")
+                    else:
+                        plt.xlabel("Z")
+                else:
+                    plt.xlabel("A")
+            if "ylabel" not in kwargs:
+                plt.ylabel("Abundance per nucleon")
+            self.apply_class_methods(plt, kwargs)
+            plt.draw()
+
+        anim = animation.FuncAnimation(fig, updatefig, y.shape[0])
+        if movie_name:
+            anim.save(movie_name, fps=fps)
+
+        return anim
+
+    def make_abundances_vs_nucleon_number_movie(
+        self,
+        movie_name="",
+        nucleon="a",
+        zone_xpath="",
+        fps=15,
+        title_func=None,
+        rcParams=None,
+        plotParams=None,
+        extraFixedCurves=None,
+        extraCurves=None,
+        **kwargs
+    ):
+        """Method to make of movie of abundances summed by nucleon number.
+
+        Args:
+
+            ``movie_name`` (:obj:`str`, optional): A string giving the name of
+            resulting movie file.
+
+            ``nucleon`` (:obj:`str`, optional): A string giving the nucleon
+            (must be 'z', 'n', or 'a').  Defaults to 'a'.
+
+            ``zone_xpath`` (:obj:`str`, optional): A string giving the XPath
+            expression to select the zones. Defaults to all zones.
+
+            ``fps`` (:obj:`float`, optional): A float giving the frames
+            per second in the resulting movie file.
+
+            ``title_func`` (optional):
+            A
+            `function \
+            <https://docs.python.org/3/library/stdtypes.html#functions>`_
+            that applies the title to each frame of the movie.  The function
+            must take a single argument, an :obj:`int` giving the index of the
+            frame to which the title will be applied.  Other data can be bound
+            to the function.  The function must return either a :obj:`str`
+            giving the title or a two-element :obj:`tuple` in which the
+            first element is a string giving the title and the second element
+            is a :obj:`dict` with optional :obj:`matplotlib.pyplot.title`
+            keyword arguments.  The default is a title giving the time in
+            seconds, the temperature in billions of Kelvins, and the
+            mass density in grams / cc.
+
+            ``rcParams`` (:obj:`dict`, optional): A dictionary of
+            :obj:`matplotlib.rcParams` to be applied to the movie.
+            Defaults to the default rcParams.
+
+            ``plotParams`` (:obj:`list`, optional): A list of
+            dictionaries of valid :obj:`matplotlib.pyplot.plot` optional
+            keyword arguments to be applied to the lines in the movie.
+
+            ``extraFixedCurves`` (:obj:`list`, optional): A list of
+            :obj:`tuple` objects giving fixed curves to appear on each
+            frame of the animation.  The first element of the tuple is a
+            :obj:`list` giving the abscissa values for the curve, the second
+            element is the ordinate values for the curve, and the third
+            element, if present, is a :obj:`dict` of
+            :obj:`matplotlib.pyplot.plot` optional keyword arguments to be
+            applied to the extra fixed curves in the movie.
+
+            ``extraCurves`` (:obj:`list`, optional): A list of
+            :obj:`tuple` objects giving curves to appear on each
+            frame of the animation.  The first element of the tuple is a
+            :obj:`list` giving the abscissa values for the curve, the second
+            element is a two-d :obj:`numpy.array` giving the ordinate values
+            for the curve corresponding to each timestep in the animation,
+            and the third element, if present, is a :obj:`dict` of
+            :obj:`matplotlib.pyplot.plot` optional keyword arguments to be
+            applied to the extra fixed curves in the movie.
+
+            ``**kwargs``:  Acceptable :obj:`matplotlib.pyplot` functions.
+            Include directly, as a :obj:`dict`, or both.
+
+        Returns:
+            The animation.
 
         """
         fig = plt.figure()
@@ -1031,18 +1254,42 @@ class Xml(wb.Base):
             ["time", "t9", "rho"], zone_xpath=zone_xpath
         )
 
+        # Check the array length against the number of steps
+
+        if extraCurves:
+            for tup in extraCurves:
+                if tup[1].shape[0] != abunds.shape[0]:
+                    print("Extra curve does not have the right length.")
+                    return None
+
         def updatefig(i):
             fig.clear()
+
             if plotParams:
                 plt.plot(abunds[i, :], **plotParams)
             else:
                 plt.plot(abunds[i, :])
+
+            if extraFixedCurves:
+                for tup in extraFixedCurves:
+                    if len(tup) == 2:
+                        plt.plot(tup[0], tup[1])
+                    else:
+                        plt.plot(tup[0], tup[1], **tup[2])
+
+            if extraCurves:
+                for tup in extraCurves:
+                    if len(tup) == 2:
+                        plt.plot(tup[0], tup[1][i])
+                    else:
+                        plt.plot(tup[0], tup[1][i], **tup[2])
+
             if title_func:
                 tf = title_func(i)
                 if tf:
-                    if type(tf) is tuple:
+                    if isinstance(tf, tuple):
                         plt.title(tf[0], **tf[1])
-                    elif type(tf) is str:
+                    elif isinstance(tf, str):
                         plt.title(tf)
                     else:
                         print("Invalid return from title function.")
@@ -1057,12 +1304,15 @@ class Xml(wb.Base):
             plt.draw()
 
         anim = animation.FuncAnimation(fig, updatefig, abunds.shape[0])
-        anim.save(movie_name, fps=fps)
+        if movie_name:
+            anim.save(movie_name, fps=fps)
+
+        return anim
 
     def make_network_abundances_movie(
         self,
-        movie_name,
-        zone_xpath=" ",
+        movie_name="",
+        zone_xpath="",
         fps=15,
         title_func=None,
         rcParams=None,
@@ -1075,14 +1325,14 @@ class Xml(wb.Base):
 
         Args:
 
-            ``movie_name`` (:obj:`str`): A string giving the name of the
-            movie to be made.
+            ``movie_name`` (:obj:`str`, optional): A string giving the name of
+            resulting movie file.
 
             ``zone_xpath`` (:obj:`str`, optional): A string giving the XPath
             expression to select the zones. Defaults to all zones.
 
             ``fps`` (:obj:`float`, optional): A float giving the frames
-            per second in the resulting movie.
+            per second in the resulting movie file.
 
             ``title_func`` (optional):
             A
@@ -1121,7 +1371,7 @@ class Xml(wb.Base):
             Include directly, as a :obj:`dict`, or both.
 
         Returns:
-            A matplotlib plot.
+            The animation.
 
         """
 
@@ -1155,7 +1405,7 @@ class Xml(wb.Base):
 
         def updatefig(i):
             fig.clear()
-            z = abunds[i, yr[0]: yr[1], xr[0]: xr[1]]
+            z = abunds[i, yr[0] : yr[1], xr[0] : xr[1]]
             plt.imshow(z, **imParams)
             if show_limits:
                 if plotParams:
@@ -1166,9 +1416,9 @@ class Xml(wb.Base):
                     plt.plot(lim["n_max"], lim["z"])
             if title_func:
                 tf = title_func(i)
-                if type(tf) is str:
+                if isinstance(tf, str):
                     plt.title(tf)
-                elif type(tf) is tuple:
+                elif isinstance(tf, tuple):
                     plt.title(tf[0], tf[1])
                 else:
                     print("Invalid return from title function.")
@@ -1183,7 +1433,10 @@ class Xml(wb.Base):
             plt.draw()
 
         anim = animation.FuncAnimation(fig, updatefig, abunds.shape[0])
-        anim.save(movie_name, fps=fps)
+        if movie_name:
+            anim.save(movie_name, fps=fps)
+
+        return anim
 
     def validate(self):
         """Method to validate the xml
@@ -1200,7 +1453,306 @@ class Xml(wb.Base):
             "reaction_data": url_prefix + "libnucnet__reac.xsd",
             "nuclear_network": url_prefix + "libnucnet__net.xsd",
             "zone_data": url_prefix + "zone_data.xsd",
-            "libnucnet_input": url_prefix + "libnucnet.xsd"}
+            "libnucnet_input": url_prefix + "libnucnet.xsd",
+        }
 
         xml_validator = etree.XMLSchema(file=xsd_dict[self._root.tag])
         xml_validator.assert_(self._xml)
+
+    def get_zone_data(self, zone_xpath=""):
+        """Method to retrieve zone data from webnucleo XML.
+
+        Args:
+            ``zone_xpath`` (:obj:`str`, optional): XPath expression to select
+            zones.  Defaults to all zones.
+
+        Returns:
+            :obj:`dict`: A dictionary of zone data.  The data for each
+            zone are themselves two :obj:`dict`, one containing properties
+            and one containing mass fractions.
+
+        """
+
+        zones = self._get_zones(zone_xpath)
+
+        result = {}
+
+        for zone in zones:
+            label = "0"
+            label_1 = zone.xpath("@label1")
+            if label_1:
+                label = label_1[0]
+            label_2 = zone.xpath("@label2")
+            if label_2:
+                label = (label, label_2[0])
+            label_3 = zone.xpath("@label3")
+            if label_3:
+                label = (label[0], label[1], label_3[0])
+            result[label] = {
+                "properties": self._get_all_zone_properties(zone),
+                "mass fractions": self._get_nuclide_data_for_zone(zone),
+            }
+
+        return result
+
+
+class New_Xml(wb.Base):
+    """A class for creating webnucleo xml files.
+
+       Each instance corresponds to new xml.  Methods set
+       the nuclide, reaction, or zone data or write the xml to a file.
+
+       Args:
+           ``xml_type`` (:obj:`str`, optional): The type of xml file to
+           be created ("nuclear_data", "reaction_data", "nuclear_network",
+           "zone_data", or "libnucnet_input").  Defaults to "nuclear_network".
+
+       """
+
+    def __init__(self, xml_type="nuclear_network"):
+        if xml_type not in [
+            "nuclear_data",
+            "reaction_data",
+            "nuclear_network",
+            "zone_data",
+            "libnucnet_input",
+        ]:
+            print("Invalid xml_type.")
+        self._root = etree.Element(xml_type)
+        self._xml = etree.ElementTree(self._root)
+        if xml_type == "nuclear_network":
+            etree.SubElement(self._root, "nuclear_data")
+            etree.SubElement(self._root, "reaction_data")
+        elif xml_type == "libnucnet_input":
+            nuclear_network = etree.SubElement(self._root, "nuclear_network")
+            etree.SubElement(nuclear_network, "nuclear_data")
+            etree.SubElement(nuclear_network, "reaction_data")
+            zone_data = etree.SubElement(self._root, "zone_data")
+
+    def _set_xml_data_for_nuclide(self, nuclide_element, nuclide):
+        states = nuclide_element.xpath("states")
+
+        if len(states) > 0:
+            states_element = states[0]
+        else:
+            etree.SubElement(nuclide_element, "z").text = str(nuclide["z"])
+            etree.SubElement(nuclide_element, "a").text = str(nuclide["a"])
+            state_element = nuclide_element
+
+        if nuclide["state"]:
+            if len(states) == 0:
+                states_element = etree.SubElement(nuclide_element, "states")
+            state_element = etree.SubElement(states_element, "state")
+            state_element.set("id", nuclide["state"])
+            etree.SubElement(state_element, "source").text = str(nuclide["source"])
+        else:
+            etree.SubElement(state_element, "source").text = str(nuclide["source"])
+
+        etree.SubElement(state_element, "mass_excess").text = str(
+            nuclide["mass excess"]
+        )
+        etree.SubElement(state_element, "spin").text = str(nuclide["spin"])
+
+        partf_element = etree.SubElement(state_element, "partf_table")
+
+        t9 = nuclide["t9"]
+        partf = nuclide["partf"]
+
+        for i in range(len(t9)):
+            point = etree.SubElement(partf_element, "point")
+            etree.SubElement(point, "t9").text = str(t9[i])
+            log10_partf = np.log10(partf[i] / (2.0 * nuclide["spin"] + 1))
+            etree.SubElement(point, "log10_partf").text = str(log10_partf)
+
+    def set_nuclide_data(self, nuclides):
+        """Method to set the nuclide data.
+
+        Args:
+
+            ``nuclides`` (:obj:`dict`): A dictionary containing the nuclides
+            to be created and their data.
+
+        Returns:
+            On successful return, the underlying xml has been created with
+            the data in ``nuclides``.
+
+        """
+
+        nuclear_data = self._xml.xpath("//nuclear_data")
+
+        if len(nuclear_data) == 0:
+            print("Attempting to set non-existent nuclear_data.")
+            return
+
+        for nuc in nuclides:
+            my_nuc = nuclides[nuc]
+            my_xpath = self._xml.xpath(
+                "//nuclear_data/nuclide[ z = "
+                + str(my_nuc["z"])
+                + " and a = "
+                + str(my_nuc["a"])
+                + "]"
+            )
+            if len(my_xpath) == 0:
+                nuclear_data[0].append(
+                    etree.Comment(
+                        self.create_nuclide_name(my_nuc["z"], my_nuc["a"], "")
+                    )
+                )
+                nuclide = etree.SubElement(nuclear_data[0], "nuclide")
+            else:
+                nuclide = my_xpath[0]
+            self._set_xml_data_for_nuclide(nuclide, nuclides[nuc])
+
+    def _set_xml_data_for_reaction(self, reaction_element, reaction):
+        etree.SubElement(reaction_element, "source").text = str(reaction.source)
+
+        for reactant in reaction.reactants:
+            etree.SubElement(reaction_element, "reactant").text = str(reactant)
+
+        for product in reaction.products:
+            etree.SubElement(reaction_element, "product").text = str(product)
+
+        if reaction.data["type"] == "single_rate":
+            etree.SubElement(reaction_element, "single_rate").text = str(
+                reaction.data["rate"]
+            )
+        elif reaction.data["type"] == "rate_table":
+            rate_table_element = etree.SubElement(reaction_element, "rate_table")
+            t9 = reaction.data["t9"]
+            rate = reaction.data["rate"]
+            sef = reaction.data["sef"]
+            for i in range(len(t9)):
+                point = etree.SubElement(rate_table_element, "point")
+                etree.SubElement(point, "t9").text = str(t9[i])
+                etree.SubElement(point, "rate").text = str(rate[i])
+                etree.SubElement(point, "sef").text = str(sef[i])
+        elif reaction.data["type"] == "non_smoker_fit":
+            nsf_element = etree.SubElement(reaction_element, "non_smoker_fit")
+            for fit in reaction.data["fits"]:
+                fit_element = etree.SubElement(nsf_element, "fit")
+                for d in fit:
+                    if d == "note":
+                        fit_element.set("note", fit[d])
+                    else:
+                        etree.SubElement(fit_element, d).text = str(fit[d])
+        else:
+            user_element = etree.SubElement(reaction_element, "user_rate")
+            user_element.set("key", reaction.data["key"])
+            properties = etree.SubElement(user_element, "properties")
+            for d in reaction.data:
+                if d != "type" and d != "key":
+                    property = etree.SubElement(properties, "property")
+                    property.text = str(reaction.data[d])
+                    if isinstance(d, tuple):
+                        property.set("name", d[0])
+                        if len(d) > 1:
+                            property.set("tag1", d[1])
+                            if len(d) > 2:
+                                property.set("tag2", d[2])
+                        if len(d) > 3:
+                            print("Improper number of property tags.")
+                            exit()
+                    else:
+                        property.set("name", d)
+
+    def set_reaction_data(self, reactions):
+        """Method to set the reaction data.
+
+        Args:
+
+            ``reactions`` (:obj:`dict`): A dictionary containing the reactions
+            to be set and their data.
+
+        Returns:
+            On successful return, the underlying xml has been created with
+            the data in ``reactions``.
+
+        """
+
+        reaction_data = self._xml.xpath("//reaction_data")
+
+        if len(reaction_data) == 0:
+            print("Attempting to set non-existent reaction_data.")
+            return
+
+        for reaction in reactions:
+            reaction_data[0].append(etree.Comment(reactions[reaction].get_string()))
+            new_reaction = etree.SubElement(reaction_data[0], "reaction")
+            self._set_xml_data_for_reaction(new_reaction, reactions[reaction])
+
+    def _set_xml_data_for_zone(self, zone_element, zone):
+
+        if len(zone["properties"]) > 0:
+            props = etree.SubElement(zone_element, "optional_properties")
+            for property in zone["properties"]:
+                prop = etree.SubElement(props, "property")
+                prop.text = str(zone["properties"][property])
+                if isinstance(property, tuple):
+                    prop.set("name", property[0])
+                    if len(property) > 1:
+                        prop.set("tag1", property[1])
+                        if len(property) > 2:
+                            prop.set("tag2", property[2])
+                            if len(property) > 3:
+                                print("Improper number of property tags.")
+                                exit()
+                else:
+                    prop.set("name", property)
+
+        mass_fracs = etree.SubElement(zone_element, "mass_fractions")
+        for nuc in zone["mass fractions"]:
+            nuclide = etree.SubElement(mass_fracs, "nuclide")
+            nuclide.set("name", nuc[0])
+            etree.SubElement(nuclide, "z").text = str(nuc[1])
+            etree.SubElement(nuclide, "a").text = str(nuc[2])
+            etree.SubElement(nuclide, "x").text = str(zone["mass fractions"][nuc])
+
+    def set_zone_data(self, zones):
+        """Method to set the zone data.
+
+        Args:
+
+            ``zones`` (:obj:`dict`): A dictionary containing the zones
+            to be set and their data.
+
+        Returns:
+            On successful return, the underlying xml has been created with
+            the data in ``reactions``.
+
+        """
+
+        zone_data = self._xml.xpath("//zone_data")
+
+        if len(zone_data) == 0:
+            print("Attempting to set non-existent reaction_data.")
+            return
+
+        for zone in zones:
+            new_zone = etree.SubElement(zone_data[0], "zone")
+            if isinstance(zone, tuple):
+                for i in range(len(zone)):
+                    label_str = "label" + str(i + 1)
+                    new_zone.set(label_str, zone[i])
+            else:
+                new_zone.set("label1", zone)
+            self._set_xml_data_for_zone(new_zone, zones[zone])
+
+    def write(self, file, pretty_print=True):
+        """Method to write the xml
+
+        Args:
+
+            ``file`` (:obj:`str`): A string giving the name of output
+            xml file.
+
+           ``pretty_print`` (:obj:`bool`, optional): If set to True,
+           routine outputs the xml in nice indented format.
+
+        Returns:
+            On successful return, the underlying xml has been written
+            to ``file``.
+
+        """
+
+        self._xml.write(file, pretty_print=pretty_print)
